@@ -37,6 +37,8 @@ static void lpmd_dump_config(lpmd_config_t *lpmd_config)
 	lpmd_log_info ("HFI LPM Enable:%d\n", lpmd_config->hfi_lpm_enable);
 	lpmd_log_info ("HFI SUV Enable:%d\n", lpmd_config->hfi_suv_enable);
 	lpmd_log_info ("WLT Hint Enable:%d\n", lpmd_config->wlt_hint_enable);
+	lpmd_log_info ("WLT Proxy Enable:%d\n", lpmd_config->wlt_proxy_enable);
+	lpmd_log_info ("WLT Proxy Enable:%d\n", lpmd_config->wlt_hint_poll_enable);
 	lpmd_log_info ("Util entry threshold:%d\n", lpmd_config->util_entry_threshold);
 	lpmd_log_info ("Util exit threshold:%d\n", lpmd_config->util_exit_threshold);
 	lpmd_log_info ("Util LP Mode CPUs:%s\n", lpmd_config->lp_mode_cpus);
@@ -59,6 +61,8 @@ static void lpmd_dump_config(lpmd_config_t *lpmd_config)
 		lpmd_log_info ("\texit_system_load_hyst:%d\n", state->exit_system_load_hyst);
 		lpmd_log_info ("\tentry_cpu_load_thres:%d\n", state->enter_cpu_load_thres);
 		lpmd_log_info ("\texit_cpu_load_thres:%d\n", state->exit_cpu_load_thres);
+		lpmd_log_info ("\tentry_gfx_load_thres:%d\n", state->enter_gfx_load_thres);
+		lpmd_log_info ("\texit_gfx_load_thres:%d\n", state->exit_gfx_load_thres);
 		lpmd_log_info ("\tWLT Type:%d\n", state->wlt_type);
 		lpmd_log_info ("\tmin_poll_interval:%d\n", state->min_poll_interval);
 		lpmd_log_info ("\tmax_poll_interval:%d\n", state->max_poll_interval);
@@ -78,6 +82,45 @@ static void lpmd_dump_config(lpmd_config_t *lpmd_config)
 	}
 }
 
+/* Set all of them Some of the operations are redundant, but it is useful to*/
+static void lpmd_init_config_state(lpmd_config_state_t *state)
+{
+	state->id = -1;
+	state->valid = 0;
+	state->name[0] = '\0';
+
+	state->wlt_type = -1;
+
+	state->entry_system_load_thres = 0;
+	state->exit_system_load_thres = 0;
+	state->exit_system_load_hyst = 0;
+	state->enter_cpu_load_thres = 0;
+	state->exit_cpu_load_thres = 0;
+	state->enter_gfx_load_thres = 0;
+	state->exit_gfx_load_thres = 0;
+
+	state->min_poll_interval = 0;
+	state->max_poll_interval = 0;
+	state->poll_interval_increment = 0;
+
+	state->epp = SETTING_IGNORE;
+	state->epb = SETTING_IGNORE;
+	state->active_cpus[0] = '\0';
+
+	state->island_0_number_p_cores = 0;
+	state->island_0_number_e_cores = 0;
+	state->island_1_number_p_cores = 0;
+	state->island_1_number_e_cores = 0;
+	state->island_2_number_p_cores = 0;
+	state->island_2_number_e_cores = 0;
+
+	state->itmt_state = SETTING_IGNORE;
+	state->irq_migrate = SETTING_IGNORE;
+
+	state->entry_load_sys = 0;
+	state->entry_load_cpu = 0;
+}
+
 static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *state)
 {
 	xmlNode *cur_node = NULL;
@@ -87,7 +130,7 @@ static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *
 	if (!doc || !a_node || !state)
 		return;
 
-	state->wlt_type = -1;
+	lpmd_init_config_state(state);
 
 	for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
 		if (cur_node->type == XML_ELEMENT_NODE) {
@@ -111,6 +154,10 @@ static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *
 					state->enter_cpu_load_thres = strtol (tmp_value, &pos, 10);
 				if (!strncmp((const char*)cur_node->name, "ExitCPULoadThres", strlen("ExitCPULoadThres")))
 					state->exit_cpu_load_thres = strtol (tmp_value, &pos, 10);
+				if (!strncmp((const char*)cur_node->name, "EnterGFXLoadThres", strlen("EnterGFXLoadThres")))
+					state->enter_gfx_load_thres = strtol (tmp_value, &pos, 10);
+				if (!strncmp((const char*)cur_node->name, "ExitGFXLoadThres", strlen("ExitGFXLoadThres")))
+					state->exit_gfx_load_thres = strtol (tmp_value, &pos, 10);
 				if (!strncmp((const char*)cur_node->name, "MinPollInterval", strlen("MinPollInterval")))
 					state->min_poll_interval = strtol (tmp_value, &pos, 10);
 				if (!strncmp((const char*)cur_node->name, "MaxPollInterval", strlen("MaxPollInterval")))
@@ -151,17 +198,30 @@ static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *
 
 static int validate_config_state(lpmd_config_t *lpmd_config, lpmd_config_state_t *state)
 {
-	if (lpmd_config->wlt_hint_enable) {
-		if (state->wlt_type >=0 && state->wlt_type <= 3)
+	if (!state->enter_gfx_load_thres && (lpmd_config->wlt_hint_enable || lpmd_config->wlt_proxy_enable)) {
+		if (state->wlt_type >=0 && state->wlt_type < WLT_INVALID)
 			state->valid = 1;
 	} else {
 		if ((state->enter_cpu_load_thres > 0 && state->enter_cpu_load_thres <= 100) ||
-		    (state->entry_system_load_thres > 0 && state->entry_system_load_thres <= 100))
+		    (state->entry_system_load_thres > 0 && state->entry_system_load_thres <= 100) ||
+		    (state->enter_gfx_load_thres > 0 && state->enter_gfx_load_thres <= 100))
 			state->valid = 1;
 	}
 	return 0;
 }
 
+static int is_wildcard(char *str)
+{
+	if (!str)
+		return 1;
+	if (strncmp(str, "*", strlen("*")))
+		return 1;
+	if (strncmp(str, " * ", strlen(" * ")))
+		return 1;
+
+	return 0;
+}
+
 static void lpmd_parse_states(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_config)
 {
 	xmlNode *cur_node = NULL;
@@ -185,15 +245,26 @@ static void lpmd_parse_states(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_
 			if (cur_node->name) {
 
 				tmp_value = (char*) xmlNodeListGetString (doc, cur_node->xmlChildrenNode, 1);
+				if (!strncmp ((const char*) cur_node->name, "CPUFamily", strlen ("CPUFamily"))) {
+					if (is_wildcard(tmp_value))
+						cpu_family = lpmd_config->cpu_family;
+					else
+						cpu_family = strtol (tmp_value, &pos, 10);
+				}
 
-				if (!strncmp ((const char*) cur_node->name, "CPUFamily", strlen ("CPUFamily")))
-					cpu_family = strtol (tmp_value, &pos, 10);
-
-				if (!strncmp ((const char*) cur_node->name, "CPUModel", strlen ("CPUModel")))
-					cpu_model = strtol (tmp_value, &pos, 10);
+				if (!strncmp ((const char*) cur_node->name, "CPUModel", strlen ("CPUModel"))) {
+					if (is_wildcard(tmp_value))
+						cpu_model = lpmd_config->cpu_model;
+					else
+						cpu_model = strtol (tmp_value, &pos, 10);
+				}
 
 				if (!strncmp ((const char*) cur_node->name, "CPUConfig", strlen ("CPUConfig"))) {
-					snprintf (cpu_config, MAX_CONFIG_LEN - 1, "%s", tmp_value);
+					if (is_wildcard(tmp_value)) {
+						strncpy(cpu_config, lpmd_config->cpu_config, MAX_CONFIG_LEN);
+					} else {
+						snprintf (cpu_config, MAX_CONFIG_LEN - 1, "%s", tmp_value);
+					}
 					cpu_config[MAX_CONFIG_LEN - 1] = '\0';
 				}
 
@@ -204,7 +275,7 @@ static void lpmd_parse_states(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_
 					continue;
 
 				/* Must check cpu family/model/config first to make sure the states applies */
-				if (cpu_family != lpmd_config->cpu_family || cpu_model != lpmd_config->cpu_model || (strncmp(cpu_config, lpmd_config->cpu_config, MAX_CONFIG_LEN) && strncmp(cpu_config, " * ", strlen(" * ")))) {
+				if (cpu_family != lpmd_config->cpu_family || cpu_model != lpmd_config->cpu_model || strncmp(cpu_config, lpmd_config->cpu_config, MAX_CONFIG_LEN)) {
 					lpmd_log_info("Ignore unsupported states for CPU family:%d,model%d,config:%s\n", cpu_family, cpu_model, cpu_config);
 					return;
 				}
@@ -263,6 +334,20 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 							|| (lpmd_config->wlt_hint_enable != 1 && lpmd_config->wlt_hint_enable != 0))
 						goto err;
 				}
+				else if (!strncmp((const char*)cur_node->name, "WLTHintPollEnable", strlen("WLtHintPollEnable"))) {
+					errno = 0;
+					lpmd_config->wlt_hint_poll_enable = strtol (tmp_value, &pos, 10);
+					if (errno || *pos != '\0'
+							|| (lpmd_config->wlt_hint_poll_enable != 1 && lpmd_config->wlt_hint_poll_enable != 0))
+						goto err;
+				}
+				else if (!strncmp((const char*)cur_node->name, "WLTProxyEnable", strlen("WLTProxyEnable"))) {
+					errno = 0;
+					lpmd_config->wlt_proxy_enable = strtol (tmp_value, &pos, 10);
+					if (errno || *pos != '\0'
+							|| (lpmd_config->wlt_proxy_enable != 1 && lpmd_config->wlt_proxy_enable != 0))
+						goto err;
+				}
 				else if (!strncmp((const char*)cur_node->name, "EntryDelayMS", strlen ("EntryDelayMS"))) {
 					errno = 0;
 					lpmd_config->util_entry_delay = strtol (tmp_value, &pos, 10);
@@ -416,6 +501,20 @@ int lpmd_get_config(lpmd_config_t *lpmd_config)
 	if (!lpmd_config)
 		return LPMD_ERROR;
 
+
+	snprintf(file_name, MAX_FILE_NAME_PATH, "%s/intel_lpmd_config_F%d_M%d_T%d.xml",
+			 TDCONFDIR, lpmd_config->cpu_family, lpmd_config->cpu_model, lpmd_config->tdp);
+
+	lpmd_log_msg ("Looking for config file %s\n", file_name);
+	if (!stat (file_name, &s))
+		goto process_xml;
+
+	snprintf(file_name, MAX_FILE_NAME_PATH, "%s/intel_lpmd_config_F%d_M%d.xml",
+			 TDCONFDIR, lpmd_config->cpu_family, lpmd_config->cpu_model);
+	lpmd_log_msg ("Looking for config file %s\n", file_name);
+	if (!stat (file_name, &s))
+		goto process_xml;
+
 	snprintf (file_name, MAX_FILE_NAME_PATH, "%s/%s", TDCONFDIR, CONFIG_FILE_NAME);
 
 	lpmd_log_msg ("Reading configuration file %s\n", file_name);
@@ -425,6 +524,7 @@ int lpmd_get_config(lpmd_config_t *lpmd_config)
 		return LPMD_ERROR;
 	}
 
+process_xml:
 	doc = xmlReadFile (file_name, NULL, 0);
 	if (doc == NULL) {
 		lpmd_log_msg ("error: could not parse file %s\n", file_name);
