@@ -164,7 +164,8 @@ int cpumask_to_str(cpu_set_t *mask, char *buf, int length)
 		}
 		offset += snprintf (buf + offset, length - 1 - offset, "%d,", i);
 	}
-	buf[offset - 1] = '\0';
+	if (offset)
+		buf[offset - 1] = '\0';
 	return 0;
 }
 
@@ -223,7 +224,7 @@ char* get_cpus_str(enum cpumask_idx idx)
 	if (cpumasks[idx].str)
 		return cpumasks[idx].str;
 
-	cpumasks[idx].str = calloc (1, MAX_STR_LENGTH);
+	cpumasks[idx].str = calloc (MAX_STR_LENGTH, 1);
 	if (!cpumasks[idx].str)
 		err (3, "STR_ALLOC");
 
@@ -242,7 +243,7 @@ static char* get_cpus_hexstr(enum cpumask_idx idx)
 	if (cpumasks[idx].hexstr)
 		return cpumasks[idx].hexstr;
 
-	cpumasks[idx].hexstr = calloc (1, MAX_STR_LENGTH);
+	cpumasks[idx].hexstr = calloc (MAX_STR_LENGTH, 1);
 	if (!cpumasks[idx].hexstr)
 		err (3, "STR_ALLOC");
 
@@ -268,7 +269,7 @@ static char* get_cpus_hexstr_reverse(enum cpumask_idx idx)
 	if (cpumasks[idx].hexstr_reverse)
 		return cpumasks[idx].hexstr_reverse;
 
-	cpumasks[idx].hexstr_reverse = calloc (1, MAX_STR_LENGTH);
+	cpumasks[idx].hexstr_reverse = calloc (MAX_STR_LENGTH, 1);
 	if (!cpumasks[idx].hexstr_reverse)
 		err (3, "STR_ALLOC");
 
@@ -305,7 +306,7 @@ static char* get_cpus_str_reverse(enum cpumask_idx idx)
 	if (cpumasks[idx].str_reverse)
 		return cpumasks[idx].str_reverse;
 
-	cpumasks[idx].str_reverse = calloc (1, MAX_STR_LENGTH);
+	cpumasks[idx].str_reverse = calloc (MAX_STR_LENGTH, 1);
 	if (!cpumasks[idx].str_reverse)
 		err (3, "STR_ALLOC");
 
@@ -576,7 +577,7 @@ int set_epp(char *path, int val, char *str)
 
 	if (val >= 0)
 		ret = fprintf (filep, "%d", val);
-	else if (str[0] != '\0')
+	else if (str && str[0] != '\0')
 		ret = fprintf (filep, "%s", str);
 	else {
 		fclose (filep);
@@ -594,6 +595,25 @@ int set_epp(char *path, int val, char *str)
 	return !(ret > 0);
 }
 
+static char *get_ppd_default_epp(void)
+{
+	int ppd_mode = get_ppd_mode();
+
+	if (ppd_mode == PPD_INVALID)
+		return NULL;
+
+	if (ppd_mode == PPD_PERFORMANCE)
+		return "performance";
+
+	if (ppd_mode == PPD_POWERSAVER)
+		return "power";
+
+	if (is_on_battery())
+		return "balance_power";
+
+	return "balance_performance";
+}
+
 int get_epp_epb(int *epp, char *epp_str, int size, int *epb)
 {
 	int c;
@@ -622,7 +642,7 @@ int init_epp_epb(void)
 	int ret;
 	char path[MAX_STR_LENGTH];
 
-	saved_cpu_info = calloc (sizeof(struct cpu_info), max_cpus);
+	saved_cpu_info = calloc (max_cpus, sizeof(struct cpu_info));
 
 	for (c = 0; c < max_cpus; c++) {
 		saved_cpu_info[c].epp_str[0] = '\0';
@@ -667,18 +687,26 @@ int process_epp_epb(void)
 
 	for (c = 0; c < max_cpus; c++) {
 		int val;
+		char *str = NULL;
 
 		if (!is_cpu_online (c))
 			continue;
 
 		if (lp_mode_epp != SETTING_IGNORE) {
-			if (lp_mode_epp == SETTING_RESTORE)
-				val = saved_cpu_info[c].epp;
-			else
+			if (lp_mode_epp == SETTING_RESTORE) {
+				val = -1;
+				str = get_ppd_default_epp();
+				if (!str) {
+					/* Fallback to cached EPP */
+					val = saved_cpu_info[c].epp;
+					str = saved_cpu_info[c].epp_str;
+				}
+			} else {
 				val = lp_mode_epp;
+			}
 
 			snprintf (path, sizeof(path), "/sys/devices/system/cpu/cpu%d/cpufreq/energy_performance_preference", c);
-			ret = set_epp (path, val, saved_cpu_info[c].epp_str);
+			ret = set_epp (path, val, str);
 			if (!ret) {
 				if (val != -1)
 					lpmd_log_debug ("Set CPU%d EPP to 0x%x\n", c, val);
@@ -853,6 +881,7 @@ static struct cpu_model_entry id_table[] = {
 		{ 6, 0xbf }, // Raptorlake S
 		{ 6, 0xaa }, // Meteorlake
 		{ 6, 0xac }, // Meteorlake
+		{ 6, 0xbd }, // Lunarlake
 		{ 0, 0 } // Last Invalid entry
 };
 
@@ -912,7 +941,7 @@ static int detect_supported_cpu(lpmd_config_t *lpmd_config)
 		return -1;
 	}
 
-	/* Platform meets all the criterias for lpmd to run, check the allow list */
+	/* Platform meets all the criteria for lpmd to run, check the allow list */
 	val = 0;
 	while (id_table[val].family) {
 		if (id_table[val].family == family && id_table[val].model == model)
@@ -1091,19 +1120,19 @@ static int is_cpu_in_l3(int cpu)
 	return 0;
 }
 
-static int is_cpu_pcore(int cpu)
+int is_cpu_pcore(int cpu)
 {
 	return !is_cpu_atom(cpu);
 }
 
-static int is_cpu_ecore(int cpu)
+int is_cpu_ecore(int cpu)
 {
 	if (!is_cpu_atom(cpu))
 		return 0;
 	return is_cpu_in_l3(cpu);
 }
 
-static int is_cpu_lcore(int cpu)
+int is_cpu_lcore(int cpu)
 {
 	if (!is_cpu_atom(cpu))
 		return 0;
@@ -1402,6 +1431,8 @@ static int restore_systemd_cgroup()
 	uint8_t *vals;
 
 	vals = calloc (size, 1);
+	if (!vals)
+		return -1;
 	get_cpus_hexvals (CPUMASK_ONLINE, vals, size);
 
 	update_allowed_cpus ("system.slice", vals, size);
@@ -1418,6 +1449,8 @@ static int update_systemd_cgroup()
 	int ret;
 
 	vals = calloc (size, 1);
+	if (!vals)
+		return -1;
 	get_cpus_hexvals (lpm_cpus_cur, vals, size);
 
 	ret = update_allowed_cpus ("system.slice", vals, size);
@@ -1917,6 +1950,8 @@ int check_cpu_capability(lpmd_config_t *lpmd_config)
 	lpmd_log_info("Detected %d Pcores, %d Ecores, %d Lcores, TDP %dW\n", pcores, ecores, lcores, tdp);
 	ret = snprintf(lpmd_config->cpu_config, MAX_CONFIG_LEN - 1, " %dP%dE%dL-%dW ", pcores, ecores, lcores, tdp);
 
+	lpmd_config->tdp = tdp;
+
 	return 0;
 }
 
