@@ -22,6 +22,8 @@
  */
 
 #include "lpmd.h"
+#include <upower.h>
+#include "wlt_proxy.h"
 
 static lpmd_config_t lpmd_config;
 
@@ -37,6 +39,8 @@ char *lpm_cmd_str[LPM_CMD_MAX] = {
 
 static int in_low_power_mode = 0;
 
+static UpClient *upower_client;
+
 static pthread_mutex_t lpmd_mutex;
 
 int lpmd_lock(void)
@@ -204,6 +208,11 @@ int in_suv_lpm(void)
 	return lpm_state & LPM_SUV_ON;
 }
 
+int in_auto_mode()
+{
+	return !(lpm_state & (LPM_USER_ON | LPM_USER_OFF));
+}
+
 /*
  * 1: request valid and already satisfied. 0: respond valid and need to continue to process. -1: request invalid
  */
@@ -550,8 +559,14 @@ static void lpmd_send_message(message_name_t msg_id, int size, unsigned char *ms
 
 void lpmd_terminate(void)
 {
+	if (lpmd_config.wlt_proxy_enable) {
+		wlt_proxy_uninit();
+	}
+
 	lpmd_send_message (TERMINATE, 0, NULL);
 	sleep (1);
+	if (upower_client)
+		g_clear_object(&upower_client);
 }
 
 void lpmd_force_on(void)
@@ -600,15 +615,6 @@ static int idx_hfi_fd = -1;
 static int wlt_fd;
 static int idx_wlt_fd = -1;
 
-/* WLT hints parsing */
-typedef enum {
-	WLT_IDLE,
-	WLT_BATTERY_LIFE,
-	WLT_SUSTAINED,
-	WLT_BURSTY,
-	WLT_INVALID,
-} wlt_type_t;
-
 // Workload type classification
 #define WORKLOAD_NOTIFICATION_DELAY_ATTRIBUTE "/sys/bus/pci/devices/0000:00:04.0/workload_hint/notification_delay_ms"
 #define WORKLOAD_ENABLE_ATTRIBUTE "/sys/bus/pci/devices/0000:00:04.0/workload_hint/workload_hint_enable"
@@ -710,6 +716,8 @@ static void poll_for_wlt(int enable)
 {
 	static int wlt_enabled_once = 0;
 
+	lpmd_log_info("%s enable:%d\n", __func__, enable);
+
 	if (wlt_fd <= 0) {
 		if (enable) {
 			wlt_fd = init_wlt();
@@ -738,6 +746,13 @@ static void poll_for_wlt(int enable)
 
 static GDBusProxy *power_profiles_daemon;
 
+static enum power_profile_daemon_mode ppd_mode = PPD_INVALID;
+
+int get_ppd_mode(void)
+{
+	return ppd_mode;
+}
+
 static void power_profiles_changed_cb(void)
 {
 	g_autoptr (GVariant)
@@ -750,14 +765,18 @@ static void power_profiles_changed_cb(void)
 
 		lpmd_log_debug ("power_profiles_changed_cb: %s\n", active_profile);
 
-		if (strcmp (active_profile, "power-saver") == 0)
+		if (strcmp (active_profile, "power-saver") == 0) {
+			ppd_mode = PPD_POWERSAVER;
 			lpmd_send_message (lpmd_config.powersaver_def, 0, NULL);
-		else if (strcmp (active_profile, "performance") == 0)
+		} else if (strcmp (active_profile, "performance") == 0) {
+			ppd_mode = PPD_PERFORMANCE;
 			lpmd_send_message (lpmd_config.performance_def, 0, NULL);
-		else if (strcmp (active_profile, "balanced") == 0)
+		} else if (strcmp (active_profile, "balanced") == 0) {
+			ppd_mode = PPD_BALANCED;
 			lpmd_send_message (lpmd_config.balanced_def, 0, NULL);
-		else
+		} else {
 			lpmd_log_warn("Ignore unsupported power profile: %s\n", active_profile);
+		}
 	}
 }
 
@@ -788,6 +807,43 @@ static void connect_to_power_profile_daemon(void)
 	}
 }
 
+static int battery_mode;
+
+int is_on_battery(void)
+{
+	return battery_mode;
+}
+
+static void upower_daemon_cb (UpClient *client, GParamSpec *pspec, gpointer user_data)
+{
+	battery_mode = up_client_get_on_battery(upower_client);
+	lpmd_log_info("upower event: on-battery: %d\n", battery_mode);
+}
+
+static void connect_to_upower_daemon(void)
+{
+	GError *error = NULL;
+	GPtrArray *devices;
+	int i;
+
+	upower_client = up_client_new_full (NULL, &error);
+	if (upower_client == NULL) {
+		g_warning ("Cannot connect to upowerd: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	lpmd_log_info("connected to upower daemon\n");
+	g_signal_connect (upower_client, "notify", G_CALLBACK (upower_daemon_cb), NULL);
+
+	devices = up_client_get_devices2 (upower_client);
+	for (i=0; i < devices->len; i++) {
+		UpDevice *device;
+		device = g_ptr_array_index (devices, i);
+		g_signal_connect (device, "notify", G_CALLBACK (upower_daemon_cb), NULL);
+	}
+}
+
 /* Poll time out default */
 #define POLL_TIMEOUT_DEFAULT_SECONDS	1
 
@@ -837,6 +893,15 @@ static int proc_message(message_capsul_t *msg)
 	return ret;
 }
 
+static void dump_poll_results(int ret)
+{
+	int i;
+
+	lpmd_log_debug("poll_fds[]: ret %d, pipe %d, uevent %d, hfi %d, wlt %d\n", ret, idx_pipe_fd, idx_uevent_fd, idx_hfi_fd, idx_wlt_fd);
+	for (i = 0; i < poll_fd_cnt; i++)
+		lpmd_log_debug("poll_fds[%d]: event %d, revent %d\n", i, poll_fds[i].events, poll_fds[i].revents);
+}
+
 // LPMD processing thread. This is callback to pthread lpmd_core_main
 static void* lpmd_core_main_loop(void *arg)
 {
@@ -853,15 +918,27 @@ static void* lpmd_core_main_loop(void *arg)
 		else if (interval == -1)
 			interval = 100;
 
+		lpmd_log_debug("Poll with interval %d\n", interval);
 		n = poll (poll_fds, poll_fd_cnt, interval);
 		if (n < 0) {
 			lpmd_log_warn ("Write to pipe failed\n");
 			continue;
 		}
+		dump_poll_results(n);
 
 		/* Time out, need to choose next util state and interval */
-		if (n == 0 && interval > 0)
-			interval = periodic_util_update (&lpmd_config, -1);
+		if (n == 0 && interval > 0) {
+			if (lpmd_config.wlt_proxy_enable) {
+				int wlt_proxy_type = read_wlt_proxy(&interval);
+				periodic_util_update (&lpmd_config, wlt_proxy_type);
+			} else if (lpmd_config.wlt_hint_enable && lpmd_config.wlt_hint_poll_enable) {
+				int wlt_type = read_wlt(wlt_fd);
+
+				interval = periodic_util_update (&lpmd_config, wlt_type);
+			} else {
+				interval = periodic_util_update (&lpmd_config, -1);
+			}
+		}
 
 		if (idx_pipe_fd >= 0 && (poll_fds[idx_pipe_fd].revents & POLLIN)) {
 //			 process message written on pipe here
@@ -891,7 +968,8 @@ static void* lpmd_core_main_loop(void *arg)
 			int wlt_index;
 
 			wlt_index = read_wlt(poll_fds[idx_wlt_fd].fd);
-			interval = periodic_util_update (&lpmd_config, wlt_index);
+			if (in_auto_mode())
+				interval = periodic_util_update (&lpmd_config, wlt_index);
 		}
 
 
@@ -980,6 +1058,7 @@ int lpmd_main(void)
 	if (ret)
 		return ret;
 
+	connect_to_upower_daemon();
 //	 Pipe is used for communication between two processes
 	ret = pipe (wake_fds);
 	if (ret) {
@@ -988,10 +1067,14 @@ int lpmd_main(void)
 	}
 	if (fcntl (wake_fds[0], F_SETFL, O_NONBLOCK) < 0) {
 		lpmd_log_error ("Cannot set non-blocking on pipe: %s\n", strerror (errno));
+		(void)close(wake_fds[0]);
+		(void)close(wake_fds[1]);
 		return LPMD_FATAL_ERROR;
 	}
 	if (fcntl (wake_fds[1], F_SETFL, O_NONBLOCK) < 0) {
 		lpmd_log_error ("Cannot set non-blocking on pipe: %s\n", strerror (errno));
+		(void)close(wake_fds[0]);
+		(void)close(wake_fds[1]);
 		return LPMD_FATAL_ERROR;
 	}
 	write_pipe_fd = wake_fds[1];
@@ -1023,9 +1106,17 @@ int lpmd_main(void)
 	}
 
 	if (lpmd_config.wlt_hint_enable) {
+		if (lpmd_config.wlt_proxy_enable) {
+			if (wlt_proxy_init() != LPMD_SUCCESS) {
+				lpmd_config.wlt_proxy_enable = 0;
+				lpmd_log_error ("Error setting up WLT Proxy. wlt_proxy_enable disabled\n");
+			}
+		}
 		if (!lpmd_config.hfi_lpm_enable && !lpmd_config.hfi_suv_enable) {
 			lpmd_config.util_enable = 0;
-			poll_for_wlt(1);
+			if (!lpmd_config.wlt_proxy_enable) {
+				poll_for_wlt(1);
+			}
 		}
 	}
 
